@@ -44,46 +44,37 @@
 #endif
 #include <ksqueezedtextlabel.h>
 
-static const QString i18n(const char *a)
-{
-    return QApplication::translate("KMessageBox", a);
-}
-static const QString i18nc(const char *comment, const char *a)
-{
-    return QApplication::translate("KMessageBox", a, comment);
-}
-
 // Some i18n filters, that standard button texts are piped through
 // (the new KGuiItem object with filtered text is created from the old one).
 
-// i18n: Filter for the Yes-button text in standard message dialogs,
+// Filter for the Yes-button text in standard message dialogs,
 // after the message caption/text have been translated.
 #define I18N_FILTER_BUTTON_YES(src, dst) \
     KGuiItem dst(src); \
-    dst.setText( i18nc( "@action:button filter-yes", src.text().toUtf8().constData() ) );
+    dst.setText( QApplication::translate( "KMessageBox", src.text().toUtf8().constData(), "@action:button filter-yes" ) );
 
-// i18n: Filter for the No-button text in standard message dialogs,
+// Filter for the No-button text in standard message dialogs,
 // after the message caption/text have been translated.
 #define I18N_FILTER_BUTTON_NO(src, dst) \
     KGuiItem dst(src); \
-    dst.setText( i18nc( "@action:button filter-no", src.text().toUtf8().constData() ) );
+    dst.setText( QApplication::translate( "KMessageBox", src.text().toUtf8().constData(), "@action:button filter-no" ) );
 
-// i18n: Filter for the Continue-button text in standard message dialogs,
+// Filter for the Continue-button text in standard message dialogs,
 // after the message caption/text have been translated.
 #define I18N_FILTER_BUTTON_CONTINUE(src, dst) \
     KGuiItem dst(src); \
-    dst.setText( i18nc( "@action:button filter-continue", src.text().toUtf8().constData() ) );
+    dst.setText( QApplication::translate( "KMessageBox", src.text().toUtf8().constData(), "@action:button filter-continue" ) );
 
-// i18n: Filter for the Cancel-button text in standard message dialogs,
+// Filter for the Cancel-button text in standard message dialogs,
 // after the message caption/text have been translated.
 #define I18N_FILTER_BUTTON_CANCEL(src, dst) \
     KGuiItem dst(src); \
-    dst.setText( i18nc( "@action:button filter-cancel", src.text().toUtf8().constData() ) );
+    dst.setText( QApplication::translate( "KMessageBox", src.text().toUtf8().constData(), "@action:button filter-cancel" ) );
 
-// i18n: Called after the button texts in standard message dialogs
+// Called after the button texts in standard message dialogs
 // have been filtered by the messages above. Not visible to user.
 #define I18N_POST_BUTTON_FILTER \
-    i18nc( "@action:button post-filter", "." );
+    QApplication::translate( "KMessageBox", ".", "@action:button post-filter" );
 
 namespace KMessageBox
 {
@@ -92,7 +83,7 @@ namespace KMessageBox
  * this static is used by the createKMessageBox function to enqueue dialogs
  * FIXME what should we do about this static?
  */
-QDialogButtonBox::StandardButton KWIDGETSADDONS_EXPORT(*KMessageBox_exec_hook)(QDialog *) = 0;
+QDialogButtonBox::StandardButton KWIDGETSADDONS_EXPORT(*KMessageBox_exec_hook)(QDialog *) = nullptr;
 
 static QIcon themedMessageBoxIcon(QMessageBox::Icon icon)
 {
@@ -111,6 +102,9 @@ static QIcon themedMessageBoxIcon(QMessageBox::Icon icon)
     case QMessageBox::Critical:
         icon_name = QStringLiteral("dialog-error");
         break;
+    case QMessageBox::Question:
+        icon_name = QStringLiteral("dialog-question");
+        break;
     default:
         break;
     }
@@ -135,6 +129,11 @@ static void applyOptions(QDialog *dialog, KMessageBox::Options options)
 // This method has been copied from KWindowSystem to avoid depending on it
 static void setMainWindow(QWidget *subWidget, WId mainWindowId)
 {
+#ifdef Q_OS_OSX
+    if (!QWidget::find(mainWindowId)) {
+        return;
+    }
+#endif
     // Set the WA_NativeWindow attribute to force the creation of the QWindow.
     // Without this QWidget::windowHandle() returns 0.
     subWidget->setAttribute(Qt::WA_NativeWindow, true);
@@ -142,6 +141,10 @@ static void setMainWindow(QWidget *subWidget, WId mainWindowId)
     Q_ASSERT(subWindow);
 
     QWindow *mainWindow = QWindow::fromWinId(mainWindowId);
+    if (!mainWindow) {
+        // foreign windows not supported on all platforms
+        return;
+    }
     // mainWindow is not the child of any object, so make sure it gets deleted at some point
     QObject::connect(subWidget, &QObject::destroyed, mainWindow, &QObject::deleteLater);
     subWindow->setTransientParent(mainWindow);
@@ -168,10 +171,10 @@ public:
         : QObject(dialog),
           m_dialog(dialog),
           m_buttons(buttons),
-          m_details(0)
+          m_details(nullptr)
     {
-        connect(m_buttons, SIGNAL(clicked(QAbstractButton*)),
-                this, SLOT(onButtonClicked(QAbstractButton*)));
+        connect(m_buttons, &QDialogButtonBox::clicked,
+                this, &DialogButtonsHelper::onButtonClicked);
     }
 
     void setDetailsWidget(QWidget *widget)
@@ -186,7 +189,7 @@ public Q_SLOTS:
         if (code != QDialogButtonBox::NoButton) {
             m_dialog->done(code);
         } else if (m_details && (button->objectName() == QStringLiteral("detailsButton"))) {
-            button->setText(i18n("&Details") + (m_details->isVisible() ? QStringLiteral(" >>") : QStringLiteral(" <<")));
+            button->setText(QApplication::translate("KMessageBox", "&Details") + (m_details->isVisible() ? QStringLiteral(" >>") : QStringLiteral(" <<")));
             m_details->setVisible(!m_details->isVisible());
         }
     }
@@ -218,6 +221,7 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
     const int spacingHint = mainWidget->style()->pixelMetric(QStyle::PM_DefaultLayoutSpacing);
     mainLayout->setSpacing(spacingHint * 2); // provide extra spacing
     mainLayout->setMargin(0);
+    buttons->setParent(dialog);
 
     QHBoxLayout *hLayout = new QHBoxLayout();
     hLayout->setMargin(0);
@@ -309,7 +313,7 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
         mainLayout->addStretch(15);
     }
 
-    QPointer<QCheckBox> checkbox = 0;
+    QPointer<QCheckBox> checkbox = nullptr;
     if (!ask.isEmpty()) {
         checkbox = new QCheckBox(ask, mainWidget);
         mainLayout->addWidget(checkbox);
@@ -323,7 +327,7 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
     topLayout->addWidget(mainWidget);
 
     if (!details.isEmpty()) {
-        QGroupBox *detailsGroup = new QGroupBox(i18n("Details"));
+        QGroupBox *detailsGroup = new QGroupBox(QApplication::translate("KMessageBox", "Details"));
         QVBoxLayout *detailsLayout = new QVBoxLayout(detailsGroup);
         if (details.length() < 512) {
             QLabel *detailsLabel = new QLabel(details);
@@ -350,7 +354,6 @@ QDialogButtonBox::StandardButton createKMessageBox(QDialog *dialog, QDialogButto
         detailsGroup->hide();
     }
 
-    buttons->setParent(dialog);
     topLayout->addWidget(buttons);
 
     if (!usingListWidget && !usingScrollArea && !usingSqueezedTextLabel && details.isEmpty()) {
@@ -486,7 +489,7 @@ static ButtonCode questionYesNoListInternal(QDialog *dialog, const QString &text
     I18N_FILTER_BUTTON_NO(buttonNo_, buttonNo)
     I18N_POST_BUTTON_FILTER
 
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Question") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Question") : caption);
     dialog->setObjectName(QStringLiteral("questionYesNo"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -497,8 +500,8 @@ static ButtonCode questionYesNoListInternal(QDialog *dialog, const QString &text
     applyOptions(dialog, options);
 
     bool checkboxResult = false;
-    const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Information, text, strlist,
-                                         dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
+    const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Question, text, strlist,
+                                         dontAskAgainName.isEmpty() ? QString() : QApplication::translate("KMessageBox", "Do not ask again"),
                                          &checkboxResult, options);
     res = (result == QDialogButtonBox::Yes ? Yes : No);
 
@@ -540,7 +543,7 @@ static ButtonCode questionYesNoCancelInternal(QDialog *dialog,
     I18N_FILTER_BUTTON_CANCEL(buttonCancel_, buttonCancel)
     I18N_POST_BUTTON_FILTER
 
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Question") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Question") : caption);
     dialog->setObjectName(QStringLiteral("questionYesNoCancel"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -552,9 +555,9 @@ static ButtonCode questionYesNoCancelInternal(QDialog *dialog,
     applyOptions(dialog, options);
 
     bool checkboxResult = false;
-    const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Information,
+    const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Question,
                                          text, QStringList(),
-                                         dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
+                                         dontAskAgainName.isEmpty() ? QString() : QApplication::translate("KMessageBox", "Do not ask again"),
                                          &checkboxResult, options);
 
     if (result == QDialogButtonBox::Yes) {
@@ -613,7 +616,7 @@ static ButtonCode warningYesNoListInternal(QDialog *dialog, const QString &text,
     I18N_FILTER_BUTTON_NO(buttonNo_, buttonNo)
     I18N_POST_BUTTON_FILTER
 
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Warning") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Warning") : caption);
     dialog->setObjectName(QStringLiteral("warningYesNoList"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -625,7 +628,7 @@ static ButtonCode warningYesNoListInternal(QDialog *dialog, const QString &text,
 
     bool checkboxResult = false;
     const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, strlist,
-                                         dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
+                                         dontAskAgainName.isEmpty() ? QString() : QApplication::translate("KMessageBox", "Do not ask again"),
                                          &checkboxResult, options);
     res = (result == QDialogButtonBox::Yes ? Yes : No);
 
@@ -676,7 +679,7 @@ static ButtonCode warningContinueCancelListInternal(QDialog *dialog, const QStri
     I18N_FILTER_BUTTON_CANCEL(buttonCancel_, buttonCancel)
     I18N_POST_BUTTON_FILTER
 
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Warning") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Warning") : caption);
     dialog->setObjectName(QStringLiteral("warningYesNo"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -688,7 +691,7 @@ static ButtonCode warningContinueCancelListInternal(QDialog *dialog, const QStri
 
     bool checkboxResult = false;
     const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, strlist,
-                                         dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
+                                         dontAskAgainName.isEmpty() ? QString() : QApplication::translate("KMessageBox", "Do not ask again"),
                                          &checkboxResult, options);
 
     if (result != QDialogButtonBox::Yes) {
@@ -744,7 +747,7 @@ static ButtonCode warningYesNoCancelListInternal(QDialog *dialog, const QString
     I18N_FILTER_BUTTON_CANCEL(buttonCancel_, buttonCancel)
     I18N_POST_BUTTON_FILTER
 
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Warning") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Warning") : caption);
     dialog->setObjectName(QStringLiteral("warningYesNoCancel"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -757,7 +760,7 @@ static ButtonCode warningYesNoCancelListInternal(QDialog *dialog, const QString
 
     bool checkboxResult = false;
     const int result = createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, strlist,
-                                         dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
+                                         dontAskAgainName.isEmpty() ? QString() : QApplication::translate("KMessageBox", "Do not ask again"),
                                          &checkboxResult, options);
 
     if (result == QDialogButtonBox::Yes) {
@@ -796,7 +799,7 @@ void error(QWidget *parent,  const QString &text,
 static void errorListInternal(QDialog *dialog, const QString &text, const QStringList &strlist,
                               const QString &caption, Options options)
 {
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Error") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Error") : caption);
     dialog->setObjectName(QStringLiteral("error"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -804,7 +807,7 @@ static void errorListInternal(QDialog *dialog, const QString &text, const QStrin
 
     applyOptions(dialog, options);
 
-    createKMessageBox(dialog, buttonBox, QMessageBox::Critical, text, strlist, QString(), 0, options);
+    createKMessageBox(dialog, buttonBox, QMessageBox::Critical, text, strlist, QString(), nullptr, options);
 }
 
 void errorList(QWidget *parent, const QString &text, const QStringList &strlist,
@@ -817,12 +820,12 @@ static void detailedErrorInternal(QDialog *dialog, const QString &text,
                                   const QString &details,
                                   const QString &caption, Options options)
 {
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Error") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Error") : caption);
     dialog->setObjectName(QStringLiteral("error"));
 
     QPushButton *detailsButton = new QPushButton;
     detailsButton->setObjectName(QStringLiteral("detailsButton"));
-    detailsButton->setText(i18n("&Details") + QStringLiteral(" >>"));
+    detailsButton->setText(QApplication::translate("KMessageBox", "&Details") + QStringLiteral(" >>"));
     detailsButton->setIcon(QIcon::fromTheme(QStringLiteral("help-about")));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -832,7 +835,7 @@ static void detailedErrorInternal(QDialog *dialog, const QString &text,
 
     applyOptions(dialog, options);
 
-    createKMessageBox(dialog, buttonBox, QMessageBox::Critical, text, QStringList(), QString(), 0, options, details);
+    createKMessageBox(dialog, buttonBox, QMessageBox::Critical, text, QStringList(), QString(), nullptr, options, details);
 }
 
 void detailedError(QWidget *parent, const QString &text,
@@ -845,7 +848,7 @@ void detailedError(QWidget *parent, const QString &text,
 static void sorryInternal(QDialog *dialog, const QString &text,
                           const QString &caption, Options options)
 {
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Sorry") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Sorry") : caption);
     dialog->setObjectName(QStringLiteral("sorry"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -853,7 +856,7 @@ static void sorryInternal(QDialog *dialog, const QString &text,
 
     applyOptions(dialog, options);
 
-    createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, QStringList(), QString(), 0, options);
+    createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, QStringList(), QString(), nullptr, options);
 }
 
 void sorry(QWidget *parent, const QString &text,
@@ -866,12 +869,12 @@ static void detailedSorryInternal(QDialog *dialog, const QString &text,
                                   const QString &details,
                                   const QString &caption, Options options)
 {
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Sorry") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Sorry") : caption);
     dialog->setObjectName(QStringLiteral("sorry"));
 
     QPushButton *detailsButton = new QPushButton;
     detailsButton->setObjectName(QStringLiteral("detailsButton"));
-    detailsButton->setText(i18n("&Details") + QStringLiteral(" >>"));
+    detailsButton->setText(QApplication::translate("KMessageBox", "&Details") + QStringLiteral(" >>"));
     detailsButton->setIcon(QIcon::fromTheme(QStringLiteral("help-about")));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -881,7 +884,7 @@ static void detailedSorryInternal(QDialog *dialog, const QString &text,
 
     applyOptions(dialog, options);
 
-    createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, QStringList(), QString(), 0, options, details);
+    createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, QStringList(), QString(), nullptr, options, details);
 }
 
 void detailedSorry(QWidget *parent, const QString &text,
@@ -905,7 +908,7 @@ static void informationListInternal(QDialog *dialog, const QString &text, const
         return;
     }
 
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Information") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Information") : caption);
     dialog->setObjectName(QStringLiteral("information"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
@@ -916,7 +919,7 @@ static void informationListInternal(QDialog *dialog, const QString &text, const
     bool checkboxResult = false;
 
     createKMessageBox(dialog, buttonBox, QMessageBox::Information, text, strlist,
-                      dontShowAgainName.isEmpty() ? QString() : i18n("Do not show this message again"),
+                      dontShowAgainName.isEmpty() ? QString() : QApplication::translate("KMessageBox", "Do not show this message again"),
                       &checkboxResult, options);
 
     if (checkboxResult) {
@@ -949,7 +952,7 @@ void about(QWidget *parent, const QString &text,
         dialog->setWindowIcon(ret);
     }
 
-    createKMessageBox(dialog, buttonBox, qApp->windowIcon(), text, QStringList(), QString(), 0, options);
+    createKMessageBox(dialog, buttonBox, qApp->windowIcon(), text, QStringList(), QString(), nullptr, options);
 }
 
 static ButtonCode messageBoxInternal(QDialog *dialog, DialogType type, const QString &text,
@@ -1129,18 +1132,18 @@ void sorryWId(WId parent_id, const QString &text,
 {
     QWidget *parent = QWidget::find(parent_id);
     QDialog *dialog = new QDialog(parent, Qt::Dialog);
-    dialog->setWindowTitle(caption.isEmpty() ? i18n("Sorry") : caption);
+    dialog->setWindowTitle(caption.isEmpty() ? QApplication::translate("KMessageBox", "Sorry") : caption);
     dialog->setObjectName(QStringLiteral("sorry"));
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(dialog);
     buttonBox->setStandardButtons(QDialogButtonBox::Ok);
 
     applyOptions(dialog, options);
-    if (parent == NULL && parent_id) {
+    if (parent == nullptr && parent_id) {
         setMainWindow(dialog, parent_id);
     }
 
-    createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, QStringList(), QString(), 0, options);
+    createKMessageBox(dialog, buttonBox, QMessageBox::Warning, text, QStringList(), QString(), nullptr, options);
 }
 
 void detailedSorryWId(WId parent_id, const QString &text,
